/*!
 * vuex-class-decorator v1.0.6
 * https://github.com/felix307253927/vuex-class
 *
 * @license
 * Copyright (c) 2017 felix
 * Released under the MIT license
 * https://github.com/felix307253927/vuex-class/blob/master/LICENSE
 */
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue'), require('vue-class-component'), require('vuex')) :
    typeof define === 'function' && define.amd ? define(['exports', 'vue', 'vue-class-component', 'vuex'], factory) :
    (factory((global.VuexClass = {}),global.Vue,global.VueClassComponent,global.Vuex));
}(this, (function (exports,Vue,VComponent,vuex) { 'use strict';

    Vue = Vue && Vue.hasOwnProperty('default') ? Vue['default'] : Vue;
    var VComponent__default = 'default' in VComponent ? VComponent['default'] : VComponent;

    var Component = VComponent__default;
    var State = createBindingHelper('computed', vuex.mapState);
    var Getter = createBindingHelper('computed', vuex.mapGetters);
    var Action = createBindingHelper('methods', vuex.mapActions);
    var Mutation = createBindingHelper('methods', vuex.mapMutations);
    function normalizeNamespace(fn) {
        return function (namespace, map, setKey) {
            if (typeof namespace !== 'string') {
                setKey = map;
                map = namespace;
                namespace = '';
            }
            else if (namespace.charAt(namespace.length - 1) !== '/') {
                namespace += '/';
            }
            return fn(namespace, map, setKey);
        };
    }
    function normalizeMap(map) {
        return Array.isArray(map)
            ? map.map(function (key) { return ({ key: key, val: key }); })
            : Object.keys(map).map(function (key) { return ({ key: key, val: map[key] }); });
    }
    function getModuleByNamespace(store, helper, namespace) {
        var module = store["_modulesNamespaceMap"][namespace];
        if (!module) {
            console.error(("[vuex] module namespace not found in " + helper + "(): " + namespace));
        }
        return module;
    }
    var GetterSetter = createBindingHelper('computed', normalizeNamespace(function (namespace, map, setKey) {
        var res = {};
        normalizeMap(map).forEach(function (ref) {
            var key = ref.key;
            var val = ref.val;
            setKey = setKey || val;
            res[key] = {
                get: function () {
                    var _key = namespace + val;
                    if (namespace && !getModuleByNamespace(this.$store, 'mapGetter', namespace)) {
                        return;
                    }
                    if (!(_key in this.$store.getters)) {
                        console.error(("[vuex] unknown getter: " + _key));
                        return;
                    }
                    return this.$store.getters[_key];
                },
                set: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    var dispatch = this.$store.dispatch;
                    if (namespace) {
                        var module = getModuleByNamespace(this.$store, 'mapAction', namespace);
                        if (!module) {
                            return;
                        }
                        dispatch = module.context.dispatch;
                    }
                    return dispatch.apply(this.$store, [setKey].concat(args));
                }
            };
        });
        return res;
    }));
    function namespace(namespace, helper) {
        function createNamespacedHelper(helper) {
            function namespacedHelper(a, b) {
                if (a instanceof Vue) {
                    var key = b;
                    var proto = a;
                    return helper(key, { namespace: namespace, setKey: key })(proto, key);
                }
                var type = a;
                var options = merge(b || {}, { namespace: namespace, setKey: b });
                return helper(type, options);
            }
            return namespacedHelper;
        }
        if (helper) {
            console.warn('[vuex-class] passing the 2nd argument to `namespace` function is deprecated. pass only namespace string instead.');
            return createNamespacedHelper(helper);
        }
        return {
            State: createNamespacedHelper(State),
            Getter: createNamespacedHelper(Getter),
            Mutation: createNamespacedHelper(Mutation),
            Action: createNamespacedHelper(Action),
            GetterSetter: createNamespacedHelper(GetterSetter)
        };
    }
    function createBindingHelper(bindTo, mapFn) {
        function makeDecorator(map, namespace, setKey) {
            return VComponent.createDecorator(function (componentOptions, key) {
                var _a;
                if (!componentOptions[bindTo]) {
                    componentOptions[bindTo] = {};
                }
                var mapObject = (_a = {}, _a[key] = map, _a);
                componentOptions[bindTo][key] = namespace !== undefined
                    ? mapFn(namespace, mapObject, setKey)[key]
                    : mapFn(mapObject, setKey)[key];
            });
        }
        function helper(a, b) {
            if (a instanceof Vue) {
                var key = b;
                var proto = a;
                return makeDecorator(key, key)(proto, key);
            }
            var namespace = extractNamespace(b);
            var type = a;
            var setKey;
            if (!b) {
                setKey = a;
            }
            else if (typeof b === 'string') {
                setKey = b;
            }
            else {
                setKey = b.setKey || a;
            }
            return makeDecorator(type, namespace, setKey);
        }
        return helper;
    }
    function extractNamespace(options) {
        if (!options || typeof options === 'string') {
            return undefined;
        }
        var n = options && options.namespace;
        if (typeof n !== 'string') {
            return undefined;
        }
        if (n[n.length - 1] !== '/') {
            return n + '/';
        }
        return n;
    }
    function merge(a, b) {
        var res = {};
        [a, b].forEach(function (obj) {
            Object.keys(obj).forEach(function (key) {
                res[key] = obj[key];
            });
        });
        return res;
    }

    exports.State = State;
    exports.Getter = Getter;
    exports.GetterSetter = GetterSetter;
    exports.Action = Action;
    exports.Mutation = Mutation;
    exports.namespace = namespace;
    exports.Component = Component;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
